"""
Meta.py
this script is responsible for introducing Meta values.
"""


class Constant:
    """
    define a constant class.
    you can create your own constant container by just instantiate this class.
    for example: `a = Constant()`
    then you can add constants under it, such as `a.NAME = 'a'`
    after that, you can't change the value of a.NAME.
    """
    def __setattr__(self, key, value):
        if key in self.__dict__:
            _error = key + ' has been defined ! '
            raise PermissionError(_error)
        self.__dict__[key] = value


class Meta:
    """
    define Meta information in math.

    *** it's strongly discouraged to instantiate this class. Meta information is expected to be uniform. ***

    # basic information
    Meta use class Constant() as its middle nodes' type,
    and Meta's end node is usually a constant data or a lambda expression or a small function.
    the general structure of Meta usually looks like this:
    Meta:
        |- CONST:
            |- PI: (float) 3.141592653589793
            |- E: (float) 2.718281828459045
            |- ...
        |- GET:
            |- ONE:
                |- int: (lambda) x: 1
                |- float: (lambda) x: 1.0
                |- complex: (lambda) x: 1 + 0j
                |- ...
            |- ZERO:
                |- int: (lambda) x: 0
                |- float: (lambda) x: 0.0
                |- complex: (lambda) x: 0 + 0j
                |- ...
            |- ANY:
                |- ...
            |- ...
        |- DETERMINE:
            |- {introduced as below}

    as above, Meta has three basic elements: CONST, GET, DETERMINE.
    CONST is used to store consistent constant in math, such as PI, E.
    GET is used to define terms in different class/type/field, such as ONE, ZERO.
    DETERMINE is used to determine if a variant is the specific value/term in specific class/type/field
    *** it's discouraged to add another basic elements unless you know well. ***

    # CONST
    if you want to add your own consistent constants, use CONST please. the statement looks like this:
    `Meta.CONST.YOUR_CONSTANT_NAME = 348236` (348236 is just a example <_<)

    # GET
    if you want to add another terms or classes under Meta.GET, please use the Constant() instantiation.
    *** classification by terms is encouraged ! ***

        for example, if you want to add a MAX as a term, you should use the following statement:
        `Meta.GET.MAX = Constant()`
        and then add its value in different class/type/field, such as:
        *** make sure the class name is correct ! ***
        `Meta.GET.MAX.int = lambda x: 999999999` (of course, it's just an example >_<)
        then you can use function `get_meta(item, _term: str, _class: type = None)` to access this value.

        alternatively, you can classify things by class/type/field, though it's not encouraged, such as:
        `Meta.YOUR_CLASS_NAME = Constant()`
        and then add its various values of different terms, such as:
        *** keep case consistent before and after ***
        `Meta.GET.YOUR_CLASS_NAME.ZERO = lambda x: YOUR_CLASS_NAME(0, x)`

    # DETERMINE
    usually, this node is not your concern.
    but if you want to specify the behavior of function `determine_meta(item, _term: str, _class: type = None)`,
    where the param {_class} or the type of param {item} is your interest class/type/field,
    you can define the value of `Meta.DETERMINE.{{term}}.{{class/type/field}}` or
    `Meta.DETERMINE.{{class/type/field}}.{{term}}`, which is usually a small function.
    by default, the function `determine_meta(item, _term: str, _class: type = None)` would return whether {item}
    is equal to `get_meta(item, _term: str, _class: type = None)`.

        for example, in class Fraction, where Meta.GET.ONE.Fraction = lambda x: Fraction(1), determine_meta(
        Fraction(2), 'ONE') is False while determine_meta(Fraction(1), 'ONE') is True.

    be careful !!!
    once the special value under your term or class is defined, it couldn't be modified,
    unless you instantiate a Constant() again.
    for example, you couldn't use `Meta.GET.MAX.int = lambda x: 100000000` after
    you had stated `Meta.GET.MAX.int = lambda x: 999999999`.
    but you could use `Meta.GET.MAX = Constant()` to redefine the Meta.GET.MAX,
    that would clear up all values of the old one.

    it's designed to protect the Meta information.
    """
    def __setattr__(self, key, value):
        self.__dict__[key] = value

    # the three basic elements
    CONST = Constant()
    GET = Constant()
    DETERMINE = Constant()

    # CONST
    CONST.PI = 3.141592653589793
    CONST.E = 2.718281828459045

    """
        classification by terms. more encouraged. 
    """
    # GET
    GET.ONE = Constant()
    GET.ZERO = Constant()
    GET.ANY = Constant()

    # DETERMINE
    DETERMINE.ONE = Constant()
    DETERMINE.ZERO = Constant()

    """
        classification by class. discouraged. 
    """
    '''
    GET.int = Constant()
    GET.int.ONE = 1
    GET.int.ZERO = 0
    
    GET.float = Constant()
    GET.float.ONE = 1.0
    GET.float.ZERO = 0.0
    
    GET.complex = Constant()
    GET.complex.ONE = 1 + 0j
    GET.complex.ZERO = 0 + 0j
    '''

    def get_meta(item: object, _term: str, _class: type = None):
        """
        get the meta information of {{_term}} in {{_class}} or type(item) if {{_class}} is None.
        *** pay attention! this function would check terms first. ***
        :param item: (any) parameter which would specify the class/type/field when _class is None
        :param _term: (str) specify the term
        :param _class: (type) specify the class/type/field
        :return: Meta.GET.{{_term}}.{{_class}}(item) or Meta.GET.{{_class}}.{{_term}}(item)
        """
        # _class
        if _class is not None:
            _class = _class.__name__
        else:
            _class = type(item).__name__
        # _term first
        if _term in Meta.GET.__dict__ and _class in Meta.GET.__dict__[_term].__dict__:
            return Meta.GET.__dict__[_term].__dict__[_class](item)
        # then _class
        if _class in Meta.GET.__dict__ and _term in Meta.GET.__dict__[_class].__dict__:
            return Meta.GET.__dict__[_class].__dict__[_term](item)
        _error = 'you have not defined Meta.GET.' + _term + '.' + _class + \
                 ' or Meta.GET.' + _class + '.' + _term + ' yet !'
        raise KeyError(_error)

    def determine_meta(item: object, _term: str, _class: type = None):
        """
        if _class is not None:
            determine if {item} is the '{_term}' in class {_class}.
        else:
            determine if {item} is the '{_term}' in class {item} belongs to.
        *** pay attention! this function would check terms first. ***
        for example: when Meta.GET.ONE.int(x) = 2 and Meta.GET.int.ONE(x) = 1, if the parameters is
            (item = 1,  _term = 'ONE', _class = int or None), the result would be False,
            since Meta.ONE.int() exists and is not equal to item.
        :param item: (any)
        :param _term: (str) specific term in class/type/field, such as ONE, ZERO, MAX, so on
        :param _class: (type) if you want to specify a specific class/type/field, use this parameter
        :return: (bool) True for yes, False for no
        """
        # _class
        if _class is not None:
            _class = _class.__name__
        else:
            _class = type(item).__name__
        # check specific DETERMINE method
        # _term first
        if _term in Meta.DETERMINE.__dict__ and _class in Meta.DETERMINE.__dict__[_term].__dict__:
            return Meta.DETERMINE.__dict__[_term].__dict__[_class](item)
        # then _class
        if _class in Meta.DETERMINE.__dict__ and _term in Meta.DETERMINE.__dict__[_class].__dict__:
            return Meta.DETERMINE.__dict__[_class].__dict__[_term](item)
        # try default method
        try:
            # _term first
            if _term in Meta.GET.__dict__ and _class in Meta.GET.__dict__[_term].__dict__:
                if item == Meta.GET.__dict__[_term].__dict__[_class](item):
                    return True
                else:
                    return False
            # then _class
            if _class in Meta.GET.__dict__ and _term in Meta.GET.__dict__[_class].__dict__:
                if item == Meta.GET.__dict__[_class].__dict__[_term](item):
                    return True
                else:
                    return False
            # if no Meta.GET.{{_term}}.{{_class}} or Meta.GET.{{_class}}.{{_term}} found
            _error = 'you have not defined Meta.GET.' + _term + '.' + _class \
                     + ' or Meta.GET.' + _class + '.' + _term + ' yet ! '
            raise KeyError(_error)
        # if Meta.GET.XXX exists, but can't be used to determine
        except ValueError:
            _error = 'the default determination method is invalid ! please define Meta.DETERMINE.' + _term + '.' + \
                     _class + ' or Meta.DETERMINE.' + _class + '.' + _term + ' specifically ! '
            raise ValueError(_error)



# __all__ = ['Meta', 'number_theory', 'fraction', 'polynomial', 'matrix']
from meta import *
from number_theory import *
from fraction import *
from polynomial import *
from matrix import *


# Meta.GET.ONE
Meta.GET.ONE.int = lambda x: 1
Meta.GET.ONE.float = lambda x: 1.0
Meta.GET.ONE.complex = lambda x: 1 + 0j
Meta.GET.ONE.Fraction = lambda x: Fraction(1)
Meta.GET.ONE.Polynomial = lambda x: Polynomial([Meta.get_meta(x.coefficient[-1], 'ONE')]) \
        if type(x).__name__ == 'Polynomial' else Polynomial([Meta.get_meta(x, 'ONE')])
Meta.GET.ONE.Matrix = lambda x: matrix_one(x.size()[0], x.size()[1], Meta.get_meta(x.kernel[-1][-1], 'ONE')) \
        if type(x).__name__ == 'Matrix' else matrix_one(1, 1, Meta.get_meta(x, 'ONE'))
# Meta.GET.ZERO
Meta.GET.ZERO.int = lambda x: 0
Meta.GET.ZERO.float = lambda x: 0.0
Meta.GET.ZERO.complex = lambda x: 0 + 0j
Meta.GET.ZERO.Fraction = lambda x: Fraction(0)
Meta.GET.ZERO.Polynomial = lambda x: Polynomial([Meta.get_meta(x.coefficient[-1], 'ZERO')]) \
        if type(x).__name__ == 'Polynomial' else Polynomial([Meta.get_meta(x, 'ZERO')])
Meta.GET.ZERO.Matrix = lambda x: matrix_zero(x.size()[0], x.size()[1], Meta.get_meta(x.kernel[-1][-1], 'ZERO')) \
        if type(x).__name__ == 'Matrix' else matrix_zero(1, 1, Meta.get_meta(x, 'ZERO'))
# Meta.GET.ANY
class ANY:
    def __str__(self):
        return 'any({0})'.format(type(self).__name__)
    def formula(self):
        return 'any({0})'.format(type(self).__name__)
    def __eq__(self, other):
        if type(self).__name__ == type(other).__name__ and str(self) == str(other):
            return True
        else:
            return False
Meta.GET.ANY.int = lambda x: type('int', (ANY, ), {})()
Meta.GET.ANY.float = lambda x: type('float', (ANY, ), {})()
Meta.GET.ANY.complex = lambda x: type('complex', (ANY, ), {})()
Meta.GET.ANY.Fraction = lambda x: type('Fraction', (ANY, ), {})()
Meta.GET.ANY.Polynomial = lambda x: type('Polynomial', (ANY, ), {})()
Meta.GET.ANY.Matrix = lambda x: type('Matrix', (ANY, ), {})()



"""
fraction.py
this script is response for the operation in fraction.
"""
from wmath.number_theory import greatest_common_divisor


class Fraction:
    """
    define the class of fraction in math and operation among them.
    """
    def __init__(self, x):
        """
        {x} accept bool, int, float, str and Fraction self type.
        for example : (True)=>1/1, (3)=>3/1, (9.3)=>93/10, ('2.0/3.6')=>5/9, (Fraction(2, 3))=>2/3
        when there are two params in {x}, which is a list or tuple,
        the first would be considered as molecule, and second as denominator.
        for example : (2, 3)=>2/3, [3.4, '3/2']=>17/75, ('4', True)=>4/1
        *** denominator can't be zero ! ***
        :param x: (bool | int | float | str | Fraction | tuple | list)
        """
        self.__dict__['__inited'] = False

        def _convert_to_fraction(_x):
            if type(_x).__name__ == 'Fraction':
                return _x.molecule, _x.denominator
            if type(_x).__name__ == 'int' or type(_x).__name__ == 'bool':
                return int(_x), 1
            _x = str(_x)
            if '/' in _x:
                _x = _x.split('/')
                _molecule_1, _denominator_1 = _convert_to_fraction(_x[0])
                _molecule_2, _denominator_2 = _convert_to_fraction(_x[1])
                _molecule, _denominator = _molecule_1 * _denominator_2, _molecule_2 * _denominator_1
                return _molecule, _denominator
            _molecule, _denominator = 1, 1
            if 'e' in _x:
                _exp = int(_x.split('e')[-1])
                if _exp < 0:
                    _denominator *= pow(10, -_exp)
                else:
                    _molecule *= pow(10, _exp)
                _x = _x.split('e')[0]
            if '.' in _x:
                _molecule *= int(_x.split('.')[0] + _x.split('.')[-1])
                _denominator *= pow(10, len(_x.split('.')[-1]))
            return _molecule, _denominator

        _type = type(x).__name__
        if _type == 'tuple' or _type == 'list':
            molecule_1, denominator_1 = _convert_to_fraction(x[0])
            molecule_2, denominator_2 = _convert_to_fraction(x[1])
            molecule, denominator = molecule_1 * denominator_2, molecule_2 * denominator_1
        elif _type == 'bool' or _type == 'int' or _type == 'float' or _type == 'str' or _type == 'Fraction':
            molecule, denominator = _convert_to_fraction(x)
        else:
            _error = 'Fraction only accepts bool | int | float | str | Fraction | tuple | list type. '
            raise ValueError(_error)

        assert type(molecule).__name__ == 'int' and type(denominator).__name__ == 'int'
        assert denominator != 0

        _greatest_common_divisor = greatest_common_divisor(molecule, denominator)
        self.molecule = molecule // _greatest_common_divisor
        self.denominator = denominator // _greatest_common_divisor
        self.__dict__['__inited'] = True

    def __getattr__(self, item):
        raise AttributeError

    def __setattr__(self, key, value):
        self.__dict__[key] = value
        if self.__dict__['__inited']:
            # dynamically adjust the fraction
            _greatest_common_divisor = greatest_common_divisor(
                self.__dict__['molecule'],
                self.__dict__['denominator'])
            self.__dict__['molecule'] = self.__dict__['molecule'] // _greatest_common_divisor
            self.__dict__['denominator'] = self.__dict__['denominator'] // _greatest_common_divisor

    def __str__(self):
        if self.denominator == 1:
            return str(self.molecule)
        else:
            return str(self.molecule) + '/' + str(self.denominator)

    def __int__(self):
        if self.denominator == 1:
            return self.molecule // self.denominator
        else:
            _error = 'can\'t convert ' + str(self) + ' to an int type. '
            raise ValueError(_error)

    def __float__(self):
        return self.molecule / self.denominator

    def __eq__(self, other):
        return self.molecule == other.molecule and self.denominator == other.denominator

    def __lt__(self, other):
        return (self - other).molecule < 0

    def __le__(self, other):
        return self == other or self < other

    def __invert__(self):
        if self.molecule == 0:
            raise ZeroDivisionError
        return Fraction([self.denominator, self.molecule])

    def __pos__(self):
        return Fraction([self.molecule, self.denominator])

    def __neg__(self):
        return Fraction([-self.molecule, self.denominator])

    def __abs__(self):
        return Fraction([abs(self.molecule), abs(self.denominator)])

    def __add__(self, other):
        _denominator = self.denominator * other.denominator
        _molecule = self.molecule * other.denominator + other.molecule * self.denominator
        return Fraction([_molecule, _denominator])

    def __sub__(self, other):
        _denominator = self.denominator * other.denominator
        _molecule = self.molecule * other.denominator - other.molecule * self.denominator
        return Fraction([_molecule, _denominator])

    def __mul__(self, other):
        _denominator = self.denominator * other.denominator
        _molecule = self.molecule * other.molecule
        return Fraction([_molecule, _denominator])

    def __truediv__(self, other):
        _denominator = self.denominator * other.molecule
        _molecule = self.molecule * other.denominator
        return Fraction([_molecule, _denominator])

    def __pow__(self, power: int or float, modulo=None):
        _denominator = pow(self.denominator, power)
        _molecule = 0
        if modulo:
            _molecule = pow(self.molecule, power, modulo * _denominator)
        else:
            _molecule = pow(self.molecule, power)
        return Fraction([_molecule, _denominator])

    def formula(self):
        """
        :return: (string) the formula form string of the fraction
        """
        if self.denominator == 1:
            return str(self.molecule)
        else:
            return '\\frac{' + str(self.molecule) + '}{' + str(self.denominator) + '}'


def list2fraction(x: list):
    """
    convert list of real numbers or number strings into list of fractions.
    it's allowed that list includes some fractions already.
    such as: [1, '1/2', Fraction(2, 3), 4]
    it's also allowed that list contains of child lists.
    such as: [1, '1/2', Fraction(2, 3), [4, 5, 6.3], -0.9]
    :param x: (list of numbers or number strings or fractions or child lists)
    :return: (list of fractions)
    """
    _list = []
    for _i in x:
        if type(_i).__name__ == 'list':
            _list.append(list2fraction(_i))
        else:
            _list.append(Fraction(_i))
    return _list


def list2str(x: list):
    """
    covert all items into strings in an any dimension list.
    it's very useful when you want to print a n dimension list while some items in it is pointers.
    :param x: (list)
    :return: (list of only strings)
    """
    _list = []
    for _i in x:
        if type(_i).__name__ == 'list':
            _list.append(list2str(_i))
        else:
            _list.append(str(_i))
    return _list


def list2float(x: list):
    """
    covert all items into float in an any dimension list.
    it's very useful when you want to convert fractions into float in a multiple dimension list.
    :param x: (list)
    :return: (list of only float)
    """
    _list = []
    for _i in x:
        if type(_i).__name__ == 'list':
            _list.append(list2float(_i))
        elif type(_i).__name__ == 'Fraction':
            _list.append(float(_i))
        elif type(_i).__name__ == 'str':
            _list.append(float(Fraction(_i)))
        else:
            _list.append(_i)
    return _list



"""
number_theory.py
this script is response for handling number theory problems in math.
"""
import math
import warnings


def is_prime(x: int):
    """
    judge weather x is a prime.
    if x <= 1, then return False.
    :param x: (int)
    :return: (bool) True if x is a prime, while False if not
    """
    if x == 2:
        return True
    if x & 1 == 0 or x <= 1:
        return False
    _is_prime = True
    _sqrt = int(math.sqrt(x)) + 1
    _i = 3
    while _i <= _sqrt:
        if x % _i == 0:
            _is_prime = False
            break
        _i += 2
    return _is_prime


def find_prime_until(x: int):
    """
    return all prime less than int x.
    :param x: (int)
    :return: (list) all prime less than int x
    """
    if x < 2:
        return []
    _prime = [2]
    _i = 3
    while _i <= x:
        _is_prime = True
        _sqrt = math.sqrt(_i)
        for _j in _prime:
            if _j > _sqrt:
                break
            if _i % _j == 0:
                _is_prime = False
                break
        if _is_prime:
            _prime.append(_i)
        _i += 2
    return _prime


def prime_factor_without_exp(x: int):
    """
    calc all prime factors of int x.
    if x is zero or one, then return [].
    if x < 0, then return the result of -x.
    :param x: (int)
    :return: (list) all prime factors of int x
    """
    x = abs(x)
    if x == 0 or x == 1:
        return []
    _list = []
    _prime = find_prime_until(int(math.sqrt(x)) + 1)
    for _i in _prime:
        if x % _i == 0:
            _list.append(_i)
            while x % _i == 0:
                x //= _i
    if x != 1:
        _list.append(x)
    return _list


def prime_factor_with_exp(x: int):
    """
    calc all prime factors and each exp of int x.
    if x is zero or one, then return {}.
    if x < 0, then return the result of -x.
    :param x: (int)
    :return: (dict) all prime factors as keys with each exp as value of int x
    """
    x = abs(x)
    if x == 0 or x == 1:
        return {}
    _dict = {}
    _sqrt = int(math.sqrt(x)) + 1
    _prime = find_prime_until(_sqrt)
    for _i in _prime:
        if x % _i == 0:
            _dict[_i] = 0
            while x % _i == 0:
                _dict[_i] += 1
                x //= _i
    if x != 1:
        _dict[x] = 1
    return _dict


def factor(x: int):
    """
    calc all factors of int x.
    if x is zero, then return [].
    if x < 0, then return the result of -x.
    :param x: (int)
    :return: (list) all factors of int x
    """
    x = abs(x)
    if x == 0:
        warnings.warn('the factors of 0 is unclear, return [] instead. ', UserWarning)
        return []
    if x == 1:
        return [1]

    def list_multiple(a: list, b: list):
        _list = []
        for _i in b:
            for _j in a:
                _list.append(_i * _j)
        return _list

    _prime_factor_with_exp = prime_factor_with_exp(x)
    _list2d = []
    for _prime in _prime_factor_with_exp:
        _list2d.append([_prime ** _i for _i in range(_prime_factor_with_exp[_prime] + 1)])
    while len(_list2d) != 1:
        _list2d[-2] = list_multiple(_list2d[-2], _list2d[-1])
        _list2d.pop()
    return _list2d[0]


def greatest_common_divisor(a: int, b: int):
    """
    calc the greatest common divisor between a and b.
    :param a: (int)
    :param b: (int)
    :return: (int) the greatest common divisor between a and b
    """
    if a == 0 and b == 0:
        warnings.warn('the greatest common divisor of 0 and 0 is unclear, return 0 instead. ', UserWarning)
        return 0
    if b == 0:
        return a
    while a % b != 0:
        _mid = b
        b = a % b
        a = _mid
    return b


def greatest_common_divisor_in_list(a: list):
    """
    calc the greatest common divisor among items in a.
    :param a: (list) integer
    :return: (int) the greatest common divisor
    """
    assert a
    if 1 in a:
        return 1
    while 0 in a:
        a.remove(0)
    if len(a) == 0:
        warnings.warn('the greatest common divisor of 0 and 0 is unclear, return 0 instead. ', UserWarning)
        return 0
    if len(a) == 1:
        return a[0]
    if len(a) == 2:
        return greatest_common_divisor(a[0], a[1])
    _list = []
    for _i in range(len(a) // 2):
        _list.append(greatest_common_divisor(a[2 * _i], a[2 * _i + 1]))
    if len(a) % 2:
        _list.append(a[-1])
    return greatest_common_divisor_in_list(_list)


def least_common_multiple(a: int, b: int):
    """
    calc the least common multiple between a and b.
    :param a: (int)
    :param b: (int)
    :return: (int) the least common multiple between a and b
    """
    return a * b // greatest_common_divisor(a, b)


def least_common_multiple_in_list(a: list):
    """
    calc the least common multiple among items in a.
    :param a: (list) integer
    :return: (int) the least common multiple
    """
    assert a
    while 1 in a:
        a.remove(1)
    if len(a) == 0:
        return 1
    if len(a) == 1:
        return a[0]
    if len(a) == 2:
        return least_common_multiple(a[0], a[1])
    _list = []
    for _i in range(len(a) // 2):
        _list.append(least_common_multiple(a[2 * _i], a[2 * _i + 1]))
    if len(a) % 2:
        _list.append(a[-1])
    return least_common_multiple_in_list(_list)


def greatest_common_divisor_with_coefficient(a: int, b: int):
    """
    calc the greatest common divisor between a and b, and find two numbers x, y to fit formula:
    a * x + b * y = the greatest common divisor.
    :param a: (int)
    :param b: (int)
    :return: (tuple) the greatest common divisor, x, y
    """
    if b == 0:
        return a, 1, 0
    else:
        __greatest_common_divisor, __x, __y = greatest_common_divisor_with_coefficient(b, a % b)
        _x = __y
        _y = __x - (a // b) * __y
        return __greatest_common_divisor, _x, _y


def inverse(a: int, n: int):
    """
    calc the inverse of a in the case of module n, where a and n must be mutually prime.
    a * x = 1 (mod n)
    :param a: (int)
    :param n: (int)
    :return: (int) x
    """
    assert greatest_common_divisor(a, n) == 1
    return greatest_common_divisor_with_coefficient(a, n)[1] % n



"""
paradigm.py
this script is response for providing a base class called Paradigm.
"""


class Paradigm:
    """
    it's base for many class related to math.
    """
    def __init__(self):
        pass

    def basic_data_type(self):
        pass

    def formula(self):
        pass



"""
polynomial.py
this script is response for the related problems in polynomial.
"""
from wmath.meta import Meta
from wmath.number_theory import (greatest_common_divisor_in_list,
                                 least_common_multiple_in_list,
                                 prime_factor_without_exp,
                                 factor)
from wmath.paradigm import Paradigm
from wmath.fraction import Fraction
from copy import deepcopy


class Polynomial(Paradigm):
    """
    define the class of polynomial and related operations among them.
    """
    def __init__(self, coefficient: list):
        super().__init__()
        assert coefficient
        _type = type(coefficient[-1]).__name__
        _coefficient = []
        for _i in coefficient:
            assert type(_i).__name__ == _type
            _coefficient.append(_i)
        while len(_coefficient) > 1 and Meta.determine_meta(_coefficient[-1], 'ZERO'):
            _coefficient.pop()
        self.coefficient = _coefficient

    def __str__(self):
        _str = '('
        for _i in range(self.degree(), 0, -1):
            _str += str(self.coefficient[_i]) + ')x**' + str(_i) + '+('
        _str += str(self.coefficient[0]) + ')'
        return _str

    def __pos__(self):
        return deepcopy(self)

    def __neg__(self):
        _coefficient = []
        for _i in self.coefficient:
            _coefficient.append(-_i)
        return Polynomial(_coefficient)

    def __eq__(self, other):
        assert self.basic_data_type().__name__ == other.basic_data_type().__name__
        if self.coefficient == other.coefficient:
            return True
        if self.degree() != other.degree():
            return False
        for _i in range(self.degree() + 1):
            if self.coefficient[_i] != other.coefficient[_i]:
                return False
        return True

    def __add__(self, other):
        assert self.basic_data_type().__name__ == other.basic_data_type().__name__
        _degree = max(self.degree(), other.degree())
        _coefficient = [Meta.get_meta(self.coefficient[-1], 'ZERO') for _i in range(_degree + 1)]
        for _i in range(_degree + 1):
            if _i <= self.degree():
                _coefficient[_i] += self.coefficient[_i]
            if _i <= other.degree():
                _coefficient[_i] += other.coefficient[_i]
        return Polynomial(_coefficient)

    def __sub__(self, other):
        assert self.basic_data_type().__name__ == other.basic_data_type().__name__
        _degree = max(self.degree(), other.degree())
        _coefficient = [Meta.get_meta(self.coefficient[-1], 'ZERO') for _i in range(_degree + 1)]
        for _i in range(_degree + 1):
            if _i <= self.degree():
                _coefficient[_i] += self.coefficient[_i]
            if _i <= other.degree():
                _coefficient[_i] -= other.coefficient[_i]
        return Polynomial(_coefficient)

    def __mul__(self, other):
        assert self.basic_data_type().__name__ == other.basic_data_type().__name__
        _degree = self.degree() + other.degree()
        _coefficient = [Meta.get_meta(self.coefficient[-1], 'ZERO') for _i in range(_degree + 1)]
        for _i in range(self.degree() + 1):
            for _j in range(other.degree() + 1):
                _coefficient[_i + _j] += self.coefficient[_i] * other.coefficient[_j]
        return Polynomial(_coefficient)

    def __truediv__(self, other):
        assert self.basic_data_type().__name__ == other.basic_data_type().__name__
        _self = +self
        if _self.degree() < other.degree():
            return Polynomial([Meta.get_meta(self.coefficient[-1], 'ZERO')]), _self
        if other.degree() == 0:
            return _self.times(Meta.get_meta(self.coefficient[-1], 'ONE') / other.coefficient[0]), \
                   Polynomial([Meta.get_meta(self.coefficient[-1], 'ZERO')])
        _degree = _self.degree() - other.degree()
        _coefficient = [Meta.get_meta(self.coefficient[-1], 'ZERO') for _i in range(_degree + 1)]
        while _self.degree() >= other.degree():
            _degree = _self.degree()
            _coefficient[_self.degree() - other.degree()] = _self.coefficient[-1] / other.coefficient[-1]
            _self -= other.times(_coefficient[_self.degree() - other.degree()], _self.degree() - other.degree(), True)
            if _self.degree() == _degree:
                _self.coefficient.pop()
        return Polynomial(_coefficient), _self

    def __floordiv__(self, other):
        return (self / other)[0]

    def __mod__(self, other):
        return (self / other)[1]

    def __pow__(self, power: int, modulo=None):
        _self = +self
        _pow = Polynomial([Meta.get_meta(self.coefficient[-1], 'ONE')])
        while power:
            if modulo:
                if power & 1:
                    _pow = _pow * _self % modulo
                _self = _self * _self % modulo
            else:
                if power & 1:
                    _pow *= _self
                _self *= _self
            power >>= 1
        return _pow

    def basic_data_type(self):
        """
        basic data type of this Polynomial.
        :return: (type)
        """
        return type(self.coefficient[-1])

    def degree(self):
        """
        degree of this polynomial.
        :return: (int)
        """
        return len(self.coefficient) - 1

    def value(self, x):
        """
        calc the value of the corresponding polynomial function where x is designated.
        :param x: (self.basic_data_type()) independent variable
        :return: (self.basic_data_type()) value
        """
        _value = Meta.get_meta(self.coefficient[-1], 'ZERO')
        for _i in range(self.degree() + 1):
            _value += self.coefficient[_i] * x ** _i
        return _value

    def monic(self, _new: bool = False):
        """
        return a monic polynomial with a same coefficient ratios of {self}.
        _new decides whether to return a new polynomial or applying change on {self}.
        :param _new: (bool)
        :return: (Polynomial) as above
        """
        if _new:
            _self = +self
        else:
            _self = self
        if Meta.determine_meta(_self.coefficient[-1], 'ONE') or \
                _self.degree() == 0 and Meta.determine_meta(_self.coefficient[0], 'ZERO'):
            return _self
        else:
            for _i in range(_self.degree()):
                _self.coefficient[_i] /= _self.coefficient[-1]
            _self.coefficient[-1] = Meta.get_meta(self.coefficient[-1], 'ONE')
            return self

    def primitive(self, _new: bool = False):
        """
        *** this function is valid only when self.basic_data_type() is Fraction ! ***
        return a primitive polynomial with a same coefficient ratios of {self}.
        _new decides whether to return a new polynomial or applying change on {self}.
        :param _new: (bool)
        :return: (Polynomial) as above
        """
        assert self.basic_data_type().__name__ == 'Fraction'
        if _new:
            _self = +self
        else:
            _self = self
        _molecule_list = []
        _denominator_list = []
        for _i in _self.coefficient:
            _molecule_list.append(_i.molecule)
            _denominator_list.append(_i.denominator)
        _molecule = least_common_multiple_in_list(_denominator_list)
        _denominator = greatest_common_divisor_in_list(_molecule_list)
        _times = Fraction([_molecule, _denominator])
        for _i in range(_self.degree() + 1):
            _self.coefficient[_i] *= _times
        return _self

    def times(self, n, degree: int = 0, _new: bool = False):
        """
        a new polynomial whose value is self * (n)x**(degree)
        _new decides whether to return the new polynomial or applying change on {self}.
        :param n: (self.basic_data_type())
        :param degree: (int)
        :param _new: (bool)
        :return: (Polynomial) the new polynomial
        """
        assert degree >= 0
        _coefficient = [Meta.get_meta(self.coefficient[-1], 'ZERO') for _i in range(degree)]
        for _i in self.coefficient:
            _coefficient.append(_i * n)
        if _new:
            return Polynomial(_coefficient)
        else:
            self.coefficient = _coefficient
            return self

    def rational_roots(self):
        """
        *** this function is valid only when self.basic_data_type() is Fraction ! ***
        calc all rational roots in the corresponding polynomial function.
        :return: (list of Fraction) all rational roots
        """
        _self = (+self).primitive()
        if _self.degree() == 0:
            return []
        if _self.degree() == 1:
            return [-_self.coefficient[0] / _self.coefficient[1]]
        _molecule_list = factor(_self.coefficient[0].molecule)
        _denominator_list = factor(_self.coefficient[-1].molecule)
        _rational_roots = []
        for _molecule in _molecule_list:
            for _denominator in _denominator_list:
                _fraction = Fraction([_molecule, _denominator])
                if _self.value(_fraction).molecule == 0:
                    _rational_roots.append(_fraction)
                if _self.value(-_fraction).molecule == 0:
                    _rational_roots.append(-_fraction)
        return _rational_roots

    def formula(self):
        """
        :return: (string) the formula form string of the polynomial
        """
        _str = ''
        if hasattr(self.basic_data_type(), 'formula'):
            for _i in range(self.degree(), 0, -1):
                _str += self.coefficient[_i].formula() + 'x^' + str(_i) + '+'
            _str += self.coefficient[0].formula()
        else:
            for _i in range(self.degree(), 0, -1):
                _str += str(self.coefficient[_i]) + 'x^' + str(_i) + '+'
            _str += str(self.coefficient[0])
        return _str

    def is_irreducible_according_eisenstein(self):
        """
        *** this function is valid only when self.basic_data_type() is Fraction ! ***
        judge weather the polynomial is irreducible according eisenstein discriminant method.
        :return: (bool) True for irreducible, and False for unclear rather than reducible
        """
        _self = (+self).primitive()
        _part_coefficient_list = [_i.molecule for _i in _self.coefficient][:-1]
        _prime_list = prime_factor_without_exp(greatest_common_divisor_in_list(_part_coefficient_list))
        if _prime_list:
            for _prime in _prime_list:
                if _self.coefficient[-1].molecule % _prime and _self.coefficient[0].molecule % _prime ** 2:
                    return True
        return False


def greatest_common_divisor_in_polynomial(a: Polynomial, b: Polynomial):
    """
    this function can figure out the greatest common divisor between a and b.
    the result polynomial is monic.
    (this function wouldn't influence the origin value of a or b although it looks like dangerous!
    this characteristic is decided by python, i have no idea. ^_^)
    :param a: (Polynomial)
    :param b: (Polynomial)
    :return: (Polynomial)
    """
    assert a.basic_data_type().__name__ == b.basic_data_type().__name__
    while (a % b).degree():
        _mid = b
        b = a % b
        a = _mid
    return b.monic()


def greatest_common_divisor_with_coefficient_in_polynomial(a: Polynomial, b: Polynomial):
    """
    calc the greatest common divisor between a and b, and find two polynomials x, y to fit formula:
    a * x + b * y = the greatest common divisor.
    :param a: (Polynomial)
    :param b: (Polynomial)
    :return: (tuple) the greatest common divisor, x, y
    """
    assert a.basic_data_type().__name__ == b.basic_data_type().__name__
    if (a % b).degree() == 0:
        _y = Polynomial([Meta.get_meta(b.coefficient[-1], 'ONE') / b.coefficient[-1]])
        return b.monic(), Polynomial([Meta.get_meta(b.coefficient[-1], 'ZERO')]), _y
    else:
        __greatest_common_divisor, __x, __y = greatest_common_divisor_with_coefficient_in_polynomial(b, a % b)
        _x = __y
        _y = __x - (a // b) * _x
        return __greatest_common_divisor, _x, _y



"""
matrix.py
this script is response for the problems related to matrix in the rational number field.
"""
from wmath.meta import Meta
from wmath.paradigm import Paradigm
from wmath.fraction import list2str
from copy import deepcopy


class Matrix(Paradigm):
    """
    define the class of matrix and related operations among them.
    """
    def __init__(self, kernel: list):
        super().__init__()
        assert kernel
        _type = type(kernel[-1][-1]).__name__
        _kernel = []
        for _i in kernel:
            assert len(_i) == len(kernel[0])
            _list = []
            for _j in _i:
                assert type(_j).__name__ == _type
                _list.append(_j)
            _kernel.append(_list)
        self.kernel = _kernel

    def __str__(self):
        _str = str(list2str(self.kernel)).replace('\'', '').replace('], [', ']\n [')
        return _str

    def __invert__(self):
        return self.inverse(_new=True)

    def __eq__(self, other):
        if self.kernel == other.kernel:
            return True
        if self.size() != other.size():
            return False
        for _i in range(self.size()[0]):
            if self.kernel[_i] == other.kernel[_i]:
                continue
            for _j in range(self.size()[1]):
                if self.kernel[_i][_j] != other.kernel[_i][_j]:
                    return False
        return True

    def __pos__(self):
        return deepcopy(self)

    def __neg__(self):
        _kernel = []
        for _i in self.kernel:
            _list = []
            for _j in _i:
                _list.append(-_j)
            _kernel.append(_list)
        return Matrix(_kernel)

    def __add__(self, other):
        assert self.basic_data_type().__name__ == other.basic_data_type().__name__
        assert self.size() == other.size()
        _kernel = []
        for _i in range(self.size()[0]):
            _kernel.append([])
            for _j in range(self.size()[1]):
                _kernel[_i].append(self.kernel[_i][_j] + other.kernel[_i][_j])
        return Matrix(_kernel)
    
    def __sub__(self, other):
        assert self.basic_data_type().__name__ == other.basic_data_type().__name__
        assert self.size() == other.size()
        _kernel = []
        for _i in range(self.size()[0]):
            _kernel.append([])
            for _j in range(self.size()[1]):
                _kernel[_i].append(self.kernel[_i][_j] - other.kernel[_i][_j])
        return Matrix(_kernel)
    
    def __mul__(self, other):
        assert self.basic_data_type().__name__ == other.basic_data_type().__name__
        assert self.size()[1] == other.size()[0]
        _len = self.size()[1]
        _kernel = [[Meta.get_meta(self.kernel[-1][-1], 'ZERO')
                    for _j in range(other.size()[1])]
                   for _i in range(self.size()[0])]
        for _i in range(self.size()[0]):
            for _j in range(other.size()[1]):
                for _k in range(_len):
                    _kernel[_i][_j] += self.kernel[_i][_k] * other.kernel[_k][_j]
        return Matrix(_kernel)

    def __truediv__(self, other):
        assert self.basic_data_type().__name__ == other.basic_data_type().__name__
        assert self.size() == other.size()
        assert self.size()[0] == self.size()[1]
        return self * other.inverse(_new=True)

    def basic_data_type(self):
        """
        basic data type of this matrix.
        :return: (type)
        """
        return type(self.kernel[-1][-1])

    def formula(self):
        """
        :return: (string) the formula form string of the matrix
        """
        _str = '\\begin{pmatrix}\n'
        if hasattr(type(self.kernel[-1][-1]), 'formula'):
            for _i in range(self.size()[0]):
                for _j in range(self.size()[1]):
                    _str += '{' + self.kernel[_i][_j].formula() + '}&'
                _str += '\\\\\n'
        else:
            for _i in range(self.size()[0]):
                for _j in range(self.size()[1]):
                    _str += '{' + str(self.kernel[_i][_j]) + '}&'
                _str += '\\\\\n'
        _str += '\\end{pmatrix}'
        return _str

    def size(self):
        """
        total number of rows and columns.
        :return: (tuple)
        """
        return len(self.kernel), len(self.kernel[0])

    def part(self, _rows, _cols):
        """
        return a new Matrix with values deep-copied from {self}, specified by {rows} and {cols}.
        _rows(_cols) accept range (_from, _to, _step) or list [a1, a2, ...] type.
        :param _rows: (range or list of int)
        :param _cols: (range or list of int)
        :return: (Matrix)
        """
        _kernel = []
        for _i in _rows:
            _list = []
            for _j in _cols:
                _list.append(deepcopy(self.kernel[_i][_j]))
            _kernel.append(_list)
        return Matrix(_kernel)

    def fill(self, _rows, _cols, other, _new=False):
        """
        fill specific part of {self} with corresponding values in {other}.
        the part is specified by {_rows} and {_cols}.
        the size of {other} must be bigger than or equal to (len(_rows), len(_cols)).
        :param _rows: (range or list of int)
        :param _cols:(range or list of int)
        :param _new: (bool) (bool) True for a new matrix, False for no
        :param other: (Matrix or list2d or tuple with the same basic_data_type of self)
        :return: (Matrix) if _new: a new matrix, else: self after filling
        """
        if _new:
            _self = +self
        else:
            _self = self
        if type(other).__name__ == 'Matrix':
            other = other.kernel
        __row, __col = 0, 0
        for _i in _rows:
            for _j in _cols:
                _self.kernel[_i][_j] = other[__row][__col]
                __col += 1
            __col = 0
            __row += 1
        return _self

    def times(self, _times, _new: bool = False, _rows=None, _cols=None):
        """
        multiply each fraction in {self} by _times.
        if _rows(_cols) is not None, it would only multiply the specific rows(cols).
        _rows(_cols) accept range (_from, _to, _step) or list [a1, a2, ...] type.
        _new decides whether to return a new matrix or applying change on {self}.
        :param _times: (self.basic_data_type()) times
        :param _new: (bool) True for a new matrix, False for no
        :param _rows: (range or list of int) keep None if you want to change all rows
        :param _cols: (range or list of int) keep None if you want to change all cols
        :return: (Matrix) if _new: a new matrix, else: self after multiplication
        """
        if _new:
            _self = +self
        else:
            _self = self
        if _rows is None:
            _rows = range(_self.size()[0])
        if _cols is None:
            _cols = range(_self.size()[1])
        for _i in _rows:
            for _j in _cols:
                _self.kernel[_i][_j] *= _times
        return _self

    def transpose(self, _new: bool = False):
        """
        transpose
        _new decides whether to return a new matrix or applying change on {self}.
        :param _new: (bool)
        :return: (Matrix) if _new: a new matrix, else: self after transpose
        """
        _kernel = []
        for _i in range(self.size()[1]):
            _kernel.append([])
            for _j in range(self.size()[0]):
                _kernel[_i].append(deepcopy(self.kernel[_j][_i]))
        if _new:
            return Matrix(_kernel)
        else:
            self.kernel = _kernel
            return self

    def stepped(self, standardized: bool = False, simplified: bool = False, _new: bool = False,
                _neg_needed: bool = False, _dependent_cols_needed: bool = False):
        """
        turn any matrix into stepped or standardized stepped or simplified stepped matrix.
        :param simplified: (bool)
        :param standardized: (bool)
        :param _new: (bool)
        :param _neg_needed: (bool)
        :param _dependent_cols_needed: (bool)
        :return: (Matrix) if _new: a new matrix, else: self after stepped or standardized stepped or simplified stepped.
            (multi) Matrix as above, [_neg: (bool) if _neg_needed], [_dependent_cols: (list) if _dependent_cols_needed]
        """
        if _new:
            _self = +self
        else:
            _self = self

        # dependent_cols and neg
        dependent_cols = []
        _row, _col = 0, 0
        _neg = False
        while _row < _self.size()[0] and _col < _self.size()[1]:

            # skip rows with a 'zero' beginning
            __row = _row
            while __row < _self.size()[0] and Meta.determine_meta(_self.kernel[__row][_col], 'ZERO'):
                __row += 1
            if __row == _self.size()[0]:
                _col += 1
                continue
            dependent_cols.append(_col)

            # upper-triangle method
            for ___row in range(__row + 1, _self.size()[0]):
                ___times = _self.kernel[___row][_col] / _self.kernel[__row][_col]
                _self.kernel[___row][_col] = Meta.get_meta(_self.kernel[___row][_col], 'ZERO')
                for ___col in range(_col + 1, _self.size()[1]):
                    _self.kernel[___row][___col] -= ___times * _self.kernel[__row][___col]

            # switch position if necessary
            if __row != _row:
                _list = _self.kernel[_row]
                _self.kernel[_row] = _self.kernel[__row]
                _self.kernel[__row] = _list
                _neg = not _neg

            _col += 1
            _row += 1

        # simplified
        if simplified:

            # independent_cols
            independent_cols = []
            for _i in range(_self.size()[1]):
                if _i not in dependent_cols:
                    independent_cols.append(_i)

            for _i in range(len(dependent_cols) - 1, -1, -1):
                for __row in range(_i):
                    __times = _self.kernel[__row][dependent_cols[_i]] / _self.kernel[_i][dependent_cols[_i]]
                    for __col in independent_cols:
                        if __col > dependent_cols[_i]:
                            _self.kernel[__row][__col] -= __times * _self.kernel[_i][__col]
                    _self.kernel[__row][dependent_cols[_i]] = Meta.get_meta(_self.kernel[-1][-1], 'ZERO')
                for __col in independent_cols:
                    if __col > dependent_cols[_i]:
                        _self.kernel[_i][__col] /= _self.kernel[_i][dependent_cols[_i]]
                _self.kernel[_i][dependent_cols[_i]] = Meta.get_meta(_self.kernel[-1][-1], 'ONE')
        else:

            # standardized
            if standardized:
                for _i in range(len(dependent_cols) - 1, -1, -1):
                    for __col in range(dependent_cols[_i] + 1, _self.size()[1]):
                        _self.kernel[_i][__col] /= _self.kernel[_i][dependent_cols[_i]]
                    _self.kernel[_i][dependent_cols[_i]] = Meta.get_meta(_self.kernel[-1][-1], 'ONE')

        if _neg_needed and _dependent_cols_needed:
            return _self, _neg, dependent_cols
        elif _neg_needed and ~_dependent_cols_needed:
            return _self, _neg
        elif ~_neg_needed and _dependent_cols_needed:
            return _self, dependent_cols
        else:
            return _self

    def rank(self):
        """
        rank of matrix.
        :return: (int) rank
        """
        _, dependent_cols = self.stepped(_new=True, _dependent_cols_needed=True)
        return len(dependent_cols)

    def determinant(self):
        """
        calc determinant of a square matrix.
        :return: (Fraction) determinant
        """
        assert self.size()[0] == self.size()[1]
        _self, _neg, _dependent_cols = self.stepped(_new=True, _neg_needed=True, _dependent_cols_needed=True)
        if len(_dependent_cols) < _self.size()[0]:
            return Meta.get_meta(_self.kernel[-1][-1], 'ZERO')
        if _neg ^ (_self.size()[0] % 2):
            _determinant = -Meta.get_meta(_self.kernel[-1][-1], 'ONE')
        else:
            _determinant = Meta.get_meta(_self.kernel[-1][-1], 'ONE')
        for _index in _dependent_cols:
            _determinant *= _self.kernel[_index][_index]
        return _determinant

    def inverse(self, _new: bool = False):
        """
        inverse
        _new decides whether to return a new matrix or applying change on {self}.
        :param _new: (bool)
        :return: (Matrix) if _new: a new matrix, else: self after inverse
        """
        assert self.size()[0] == self.size()[1]
        _kernel = (+self).kernel
        _len = len(_kernel)

        # create _inverse as result
        _inverse = [[Meta.get_meta(_kernel[-1][-1], 'ZERO') for _j in range(_len)] for _i in range(_len)]
        for _i in range(_len):
            _inverse[_i][_i] = Meta.get_meta(_kernel[-1][-1], 'ONE')

        # upper triangle
        for _index in range(_len):

            # skip rows with a 'zero' beginning
            _row = _index
            while _row < _len and Meta.determine_meta(_kernel[_row][_index], 'ZERO'):
                _row += 1
            if _row == _len:
                return None

            for __row in range(_row + 1, _len):
                __times = _kernel[__row][_index] / _kernel[_row][_index]
                for __col in range(_index + 1, _len):
                    _kernel[__row][__col] -= __times * _kernel[_row][__col]
                for __col in range(_len):
                    _inverse[__row][__col] -= __times * _inverse[_row][__col]

            # switch position if necessary
            if _row != _index:
                _list = _kernel[_index]
                _kernel[_index] = _kernel[_row]
                _kernel[_row] = _list
                _list = _inverse[_index]
                _inverse[_index] = _inverse[_row]
                _inverse[_row] = _list

        # lower triangle
        for _index in range(_len - 1, -1, -1):
            for _row in range(_index):
                _times = _kernel[_row][_index] / _kernel[_index][_index]
                for _col in range(_len):
                    _inverse[_row][_col] -= _times * _inverse[_index][_col]
            for _col in range(_len):
                _inverse[_index][_col] /= _kernel[_index][_index]

        if _new:
            return Matrix(_inverse)
        else:
            self.kernel = _inverse
            return self

    def accompany(self):
        """
        accompany matrix
        :return: (Matrix) if _new: a new matrix, else: self after turning to its accompany matrix
        """
        assert self.size()[0] == self.size()[1]
        _inverse = self.inverse(_new=True)
        if _inverse is not None:
            _determinant = self.determinant()
            return _inverse.times(_determinant)
        if self.rank() < self.size()[0] - 1:
            return matrix_zero(self.size()[0], self.size()[1], Meta.get_meta(self.kernel[-1][-1], 'ZERO'))
        _kernel = []
        for _i in range(self.size()[0]):
            _kernel.append([])
            for _j in range(self.size()[1]):
                _i_list = [__i for __i in range(self.size()[0]) if __i != _i]
                _j_list = [__j for __j in range(self.size()[0]) if __j != _j]
                _kernel[_i].append(self.part(_i_list, _j_list).determinant())
        return Matrix(_kernel)


def matrix_zero(_row: int, _col: int, _filled):
    """
    return a matrix filled with {_filled}, with a size (_row, _col).
    :param _row: (int)
    :param _col: (int)
    :param _filled: (any)
    :return: (Matrix)
    """
    _kernel = [[_filled for _j in range(_col)] for _i in range(_row)]
    return Matrix(_kernel)


def matrix_one(_row: int, _col: int, _value):
    """
    return a 'E' matrix with {_value} on the diagonal, with a size (_row, _col).
    :param _row: (int)
    :param _col: (int)
    :param _value: (any)
    :return: (Matrix)
    """
    _kernel = [[Meta.get_meta(_value, 'ZERO') for _j in range(_col)] for _i in range(_row)]
    for _i in range(min(_row, _col)):
        _kernel[_i][_i] = _value
    return Matrix(_kernel)


def matrix_horizontal_stack(matrices: list):
    """
    stack matrices horizontally.
    :param matrices: (list of Matrix)
    :return: (Matrix)
    """
    assert matrices
    for _i in range(1, len(matrices)):
        assert matrices[_i].basic_data_type() == matrices[0].basic_data_type()
        assert matrices[_i].size()[0] == matrices[0].size()[0]
    _matrices = deepcopy(matrices)
    _kernel = []
    for _i in range(_matrices[0].size()[0]):
        _kernel.append([])
        for _j in range(len(_matrices)):
            for _k in range(_matrices[_j].size()[1]):
                _kernel[_i].append(_matrices[_j].kernel[_i][_k])
    return Matrix(_kernel)


def matrix_vertical_stack(matrices: list):
    """
    stacking two matrices vertically.
    :param matrices: (list of Matrix)
    :return: (Matrix)
    """
    assert matrices
    for _i in range(1, len(matrices)):
        assert matrices[_i].basic_data_type() == matrices[0].basic_data_type()
        assert matrices[_i].size()[1] == matrices[0].size()[1]
    _matrices = deepcopy(matrices)
    _kernel = []
    _row = 0
    for _i in range(len(_matrices)):
        for _j in range(_matrices[_i].size()[0]):
            _kernel.append([])
            for _k in range(_matrices[0].size()[1]):
                _kernel[_row + _j].append(_matrices[_i].kernel[_j][_k])
        _row += _matrices[_i].size()[0]
    return Matrix(_kernel)


def homogeneous_linear_equations(a: Matrix):
    """
    figure out the fundamental system of solutions of homogeneous linear equations: a * X = Matrix(zero).
    :param a: (Matrix) as above
    :return: (list of Matrix) fundamental system of solutions
    """
    # upper triangle
    _a, dependent_cols = a.stepped(simplified=True, _new=True, _dependent_cols_needed=True)
    # fundamental solutions
    independent_cols = []
    for _i in range(_a.size()[1]):
        if _i not in dependent_cols:
            independent_cols.append(_i)
    _len = len(independent_cols)
    _fundamental_solutions_matrix = matrix_zero(_a.size()[1], _len, Meta.get_meta(_a.kernel[-1][-1], 'ZERO'))
    _fundamental_solutions_matrix.fill(dependent_cols, range(_len),
                                       (-_a.part(range(_a.size()[0]), independent_cols)).kernel)
    _fundamental_solutions_matrix.fill(independent_cols, range(_len),
                                       matrix_one(_len, _len, Meta.get_meta(_a.kernel[-1][-1], 'ONE')).kernel)
    _fundamental_solutions_list = [_fundamental_solutions_matrix.part(range(_a.size()[1]), [_i]) for _i in range(_len)]
    return _fundamental_solutions_list


def non_homogeneous_linear_equations(a: Matrix, b: Matrix):
    """
    figure out the fundamental system of solutions and one special solution of non homogeneous linear equations:
    a * X = b.
    good news ! argument {b} could be a multi-columns matrix, which means this function can solve multiple equations at
    the same time.
    :param a: (Matrix) as above
    :param b: (Matrix) as above
    :return: (list of fundamental solutions, list of special solutions)
    """
    assert a.basic_data_type().__name__ == b.basic_data_type().__name__
    assert a.size()[0] == b.size()[0]
    _a = matrix_horizontal_stack([a, b])
    _a, dependent_cols = _a.stepped(simplified=True, _dependent_cols_needed=True)
    # fundamental solutions
    _dependent_cols = []
    _independent_cols = []
    for _i in range(a.size()[1]):
        if _i in dependent_cols:
            _dependent_cols.append(_i)
        else:
            _independent_cols.append(_i)
    _rank = len(_dependent_cols)
    _len = len(_independent_cols)
    if _len == 0:
        _fundamental_solutions_list = []
    else:
        _fundamental_solutions_matrix = matrix_zero(a.size()[1], _len, Meta.get_meta(_a.kernel[-1][-1], 'ZERO'))
        _fundamental_solutions_matrix.fill(_dependent_cols, range(_len),
                                           (-_a.part(range(_a.size()[0]), _independent_cols)).kernel)
        _fundamental_solutions_matrix.fill(_independent_cols, range(_len),
                                           matrix_one(_len, _len, Meta.get_meta(_a.kernel[-1][-1], 'ONE')).kernel)
        _fundamental_solutions_list = [_fundamental_solutions_matrix.part(range(a.size()[1]), [i]) for i in range(_len)]
    # special solutions
    _special_solutions_list = []
    for _i in range(b.size()[1]):
        _row = b.size()[0] - 1
        while Meta.determine_meta(_a.kernel[_row][a.size()[1] + _i], 'ZERO'):
            _row -= 1
        if _row < _rank:
            _special_solution = matrix_zero(a.size()[1], 1, Meta.get_meta(_a.kernel[-1][-1], 'ZERO'))
            _special_solution.fill(_dependent_cols, [0], _a.part(range(_a.size()[0]), [a.size()[1] + _i]).kernel)
            _special_solutions_list.append(_special_solution)
        else:
            _special_solutions_list.append(None)
    return _fundamental_solutions_list, _special_solutions_list
